@@ -268,7 +268,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.1.1';
 
 my ( $d_sane, $logger );
 my $tolerance             = 1;
@@ -642,7 +642,7 @@ sub _save_profile_callback {
     my $entry = Gtk3::Entry->new;
     $entry->set_activates_default(TRUE);
     $hbox->pack_end( $entry, TRUE, TRUE, 0 );
-    $dialog->vbox->add($hbox);
+    $dialog->get_content_area->add($hbox);
     $dialog->set_default_response('ok');
     $dialog->show_all;
     my $flag = TRUE;
@@ -1237,7 +1237,6 @@ sub update_options {
     # set.
     my $current_scan_options = dclone( $self->{current_scan_options} );
 
-
     # walk the widget tree and update them from the hash
     my $num_dev_options = $new_options->num_options;
     my $options         = $self->get('available-scan-options');
@@ -2214,7 +2213,7 @@ sub _set_option_profile {
                 my ( $widget, $optname, $optval, $uuid ) = @_;
 
                 # With multiple reloads, this can get called several times,
-                # so only react to to signal from the correct profile
+                # so only react to signal from the correct profile
                 if ( defined $uuid and $uuid eq $profile->{uuid} ) {
                     $self->signal_handler_disconnect($signal);
                     $self->_set_option_profile( $profile, $next );
